@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_i960.c,v 1.11 2006/07/20 21:52:59 debug Exp $
+ *  $Id: cpu_i960.c,v 1.12 2006/09/19 10:50:08 debug Exp $
  *
  *  Intel i960 CPU emulation.
  */
@@ -39,6 +39,7 @@
 #include "machine.h"
 #include "memory.h"
 #include "misc.h"
+#include "settings.h"
 #include "symbol.h"
 
 #define	DYNTRANS_32
@@ -75,6 +76,9 @@ int i960_cpu_new(struct cpu *cpu, struct memory *mem, struct machine *machine,
 		debug("%s", cpu->name);
 	}
 
+	/*  Add all register names to the settings:  */
+	CPU_SETTINGS_ADD_REGISTER64("pc", cpu->pc);
+
 	return 1;
 }
 
@@ -126,29 +130,6 @@ void i960_cpu_register_dump(struct cpu *cpu, int gprs, int coprocs)
 }
 
 
-/*
- *  i960_cpu_register_match():
- */
-void i960_cpu_register_match(struct machine *m, char *name,
-	int writeflag, uint64_t *valuep, int *match_register)
-{
-	int cpunr = 0;
-
-	/*  CPU number:  */
-
-	/*  TODO  */
-
-	/*  Register name:  */
-	if (strcasecmp(name, "pc") == 0) {
-		if (writeflag) {
-			m->cpus[cpunr]->pc = *valuep;
-		} else
-			*valuep = m->cpus[cpunr]->pc;
-		*match_register = 1;
-	}
-}
-
-
 /*
  *  i960_cpu_tlbdump():
  *
