@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_mips_coproc.c,v 1.53 2006/08/11 17:43:30 debug Exp $
+ *  $Id: cpu_mips_coproc.c,v 1.60 2006/10/14 23:47:37 debug Exp $
  *
  *  Emulation of MIPS coprocessors.
  */
@@ -45,6 +45,7 @@
 #include "mips_cpu_types.h"
 #include "misc.h"
 #include "opcodes_mips.h"
+#include "timer.h"
 
 
 #ifndef ENABLE_MIPS
@@ -437,6 +438,23 @@ struct mips_coproc *mips_coproc_new(struct cpu *cpu, int coproc_nr)
 }
 
 
+/*
+ *  mips_timer_tick():
+ */
+static void mips_timer_tick(struct timer *timer, void *extra)
+{
+	struct cpu *cpu = (struct cpu *) extra;
+
+	cpu->cd.mips.compare_interrupts_pending ++;
+
+	if ((int32_t) (cpu->cd.mips.coproc[0]->reg[COP0_COUNT] -
+	    cpu->cd.mips.coproc[0]->reg[COP0_COMPARE]) < 0) {
+		cpu->cd.mips.coproc[0]->reg[COP0_COUNT] =
+		    cpu->cd.mips.coproc[0]->reg[COP0_COMPARE];
+	}
+}
+
+
 /*
  *  mips_coproc_tlb_set_entry():
  *
@@ -595,7 +613,11 @@ void coproc_register_read(struct cpu *cpu,
 	if (cp->coproc_nr==0 && reg_nr==COP0_PAGEMASK)	unimpl = 0;
 	if (cp->coproc_nr==0 && reg_nr==COP0_WIRED)	unimpl = 0;
 	if (cp->coproc_nr==0 && reg_nr==COP0_BADVADDR)	unimpl = 0;
-	if (cp->coproc_nr==0 && reg_nr==COP0_COUNT)	unimpl = 0;
+	if (cp->coproc_nr==0 && reg_nr==COP0_COUNT) {
+		/*  TODO: Increase count in a more meaningful way!  */
+		cp->reg[COP0_COUNT] = (int32_t) (cp->reg[COP0_COUNT] + 1);
+		unimpl = 0;
+	}
 	if (cp->coproc_nr==0 && reg_nr==COP0_ENTRYHI)	unimpl = 0;
 	if (cp->coproc_nr==0 && reg_nr==COP0_COMPARE)	unimpl = 0;
 	if (cp->coproc_nr==0 && reg_nr==COP0_STATUS)	unimpl = 0;
@@ -753,13 +775,49 @@ void coproc_register_write(struct cpu *cpu,
 			unimpl = 0;
 			break;
 		case COP0_COMPARE:
-			/*  Clear the timer interrupt bit (bit 7):  */
-			cpu->cd.mips.compare_register_set = 1;
+			if (cpu->machine->emulated_hz > 0) {
+				int32_t compare_diff = tmp -
+				    cp->reg[COP0_COMPARE];
+				double hz;
+
+				if (compare_diff < 0)
+					hz = tmp - cp->reg[COP0_COUNT];
+
+				if (compare_diff == 0)
+					hz = 0;
+				else
+					hz = (double)cpu->machine->emulated_hz
+					    / (double)compare_diff;
+/*
+ *  TODO: DON'T HARDCODE THIS!
+ */
+hz = 100.0;
+
+				/*  Initialize or re-set the periodic timer:  */
+				if (hz > 0) {
+					if (cpu->cd.mips.timer == NULL)
+						cpu->cd.mips.timer = timer_add(
+						    hz, mips_timer_tick, cpu);
+					else
+						timer_update_frequency(
+						    cpu->cd.mips.timer, hz);
+				}
+			}
+
+			/*  Ack the periodic timer, if it was asserted:  */
+			if (cp->reg[COP0_CAUSE] & 0x8000 &&
+			    cpu->cd.mips.compare_interrupts_pending > 0)
+				cpu->cd.mips.compare_interrupts_pending --;
+
+			/*  Clear the timer interrupt assertion (bit 7):  */
 			mips_cpu_interrupt_ack(cpu, 7);
+
 			if (tmp != (uint64_t)(int64_t)(int32_t)tmp)
 				fatal("WARNING: trying to write a 64-bit value"
 				    " to the COMPARE register!\n");
+
 			tmp = (int64_t)(int32_t)tmp;
+			cpu->cd.mips.compare_register_set = 1;
 			unimpl = 0;
 			break;
 		case COP0_ENTRYHI:
@@ -1622,29 +1680,28 @@ void coproc_tlbwri(struct cpu *cpu, int randomflag)
 		    (cp->reg[COP0_ENTRYHI] & R2K3K_ENTRYHI_ASID_MASK) ))
 			cpu->invalidate_translation_caches(cpu, oldvaddr,
 			    INVALIDATE_VADDR);
+
 		break;
 
 	default:if (cpu->cd.mips.cpu_type.mmu_model == MMU10K) {
-			oldvaddr = cp->tlbs[index].hi & ENTRYHI_VPN2_MASK_R10K;
+			oldvaddr = cp->tlbs[index].hi &
+			    (ENTRYHI_VPN2_MASK_R10K | ENTRYHI_R_MASK);
 			/*  44 addressable bits:  */
 			if (oldvaddr & 0x80000000000ULL)
-				oldvaddr |= 0xfffff00000000000ULL;
+				oldvaddr |= 0x3ffff00000000000ULL;
 		} else if (cpu->is_32bit) {
 			/*  MIPS32 etc.:  */
 			oldvaddr = cp->tlbs[index].hi & ENTRYHI_VPN2_MASK;
 			oldvaddr = (int32_t)oldvaddr;
 		} else {
 			/*  Assume MMU4K  */
-			oldvaddr = cp->tlbs[index].hi & ENTRYHI_VPN2_MASK;
+			oldvaddr = cp->tlbs[index].hi &
+			    (ENTRYHI_R_MASK | ENTRYHI_VPN2_MASK);
 			/*  40 addressable bits:  */
 			if (oldvaddr & 0x8000000000ULL)
-				oldvaddr |= 0xffffff0000000000ULL;
+				oldvaddr |= 0x3fffff0000000000ULL;
 		}
 
-#if 0
-		/*  TODO: FIX THIS! It shouldn't be needed!  */
-		cpu->invalidate_translation_caches(cpu, 0, INVALIDATE_ALL);
-#else
 		/*
 		 *  TODO: non-4KB page sizes!
 		 */
@@ -1654,7 +1711,6 @@ void coproc_tlbwri(struct cpu *cpu, int randomflag)
 		if (cp->tlbs[index].lo1 & ENTRYLO_V)
 			cpu->invalidate_translation_caches(cpu, oldvaddr|0x1000,
 			    INVALIDATE_VADDR);
-#endif
 	}
 
 #if 0
@@ -1672,7 +1728,8 @@ void coproc_tlbwri(struct cpu *cpu, int randomflag)
 		int i;
 		unsigned int asid;
 
-		vaddr1 = cp->reg[COP0_ENTRYHI] & ENTRYHI_VPN2_MASK_R10K;
+		vaddr1 = cp->reg[COP0_ENTRYHI] &
+		    (ENTRYHI_R_MASK | ENTRYHI_VPN2_MASK_R10K);
 		asid = cp->reg[COP0_ENTRYHI] & ENTRYHI_ASID;
 		/*  Since this is just a warning, it's probably not necessary
 		    to use R4000 masks etc.  */
@@ -1685,7 +1742,8 @@ void coproc_tlbwri(struct cpu *cpu, int randomflag)
 			    (cp->tlbs[i].hi & ENTRYHI_ASID) != asid)
 				continue;
 
-			vaddr2 = cp->tlbs[i].hi & ENTRYHI_VPN2_MASK_R10K;
+			vaddr2 = cp->tlbs[i].hi &
+			    (ENTRYHI_R_MASK | ENTRYHI_VPN2_MASK_R10K);
 			if (vaddr1 == vaddr2 && ((cp->tlbs[i].lo0 &
 			    ENTRYLO_V) || (cp->tlbs[i].lo1 & ENTRYLO_V)))
 				fatal("\n[ WARNING! tlbw%s to index 0x%02x "
@@ -1719,6 +1777,9 @@ void coproc_tlbwri(struct cpu *cpu, int randomflag)
 			    INVALIDATE_PADDR);
 		}
 
+		/*  Set new last_written_tlb_index hint:  */
+		cpu->cd.mips.last_written_tlb_index = index;
+
 		if (cp->reg[COP0_STATUS] & MIPS1_ISOL_CACHES) {
 			fatal("Wow! Interesting case; tlbw* while caches"
 			    " are isolated. TODO\n");
@@ -1782,20 +1843,22 @@ void coproc_tlbwri(struct cpu *cpu, int randomflag)
 		    >> ENTRYLO_PFN_SHIFT) << pfn_shift;
 
 		if (cpu->cd.mips.cpu_type.mmu_model == MMU10K) {
-			vaddr0 = cp->tlbs[index].hi & ENTRYHI_VPN2_MASK_R10K;
+			vaddr0 = cp->tlbs[index].hi &
+			    (ENTRYHI_R_MASK | ENTRYHI_VPN2_MASK_R10K);
 			/*  44 addressable bits:  */
 			if (vaddr0 & 0x80000000000ULL)
-				vaddr0 |= 0xfffff00000000000ULL;
+				vaddr0 |= 0x3ffff00000000000ULL;
 		} else if (cpu->is_32bit) {
 			/*  MIPS32 etc.:  */
 			vaddr0 = cp->tlbs[index].hi & ENTRYHI_VPN2_MASK;
 			vaddr0 = (int32_t)vaddr0;
 		} else {
 			/*  Assume MMU4K  */
-			vaddr0 = cp->tlbs[index].hi & ENTRYHI_VPN2_MASK;
+			vaddr0 = cp->tlbs[index].hi &
+			    (ENTRYHI_R_MASK | ENTRYHI_VPN2_MASK);
 			/*  40 addressable bits:  */
 			if (vaddr0 & 0x8000000000ULL)
-				vaddr0 |= 0xffffff0000000000ULL;
+				vaddr0 |= 0x3fffff0000000000ULL;
 		}
 
 		vaddr1 = vaddr0 | (1 << vpn_shift);
@@ -1847,6 +1910,9 @@ void coproc_tlbwri(struct cpu *cpu, int randomflag)
 		if (memblock != NULL && cp->reg[COP0_ENTRYLO1] & ENTRYLO_V)
 			cpu->update_translation_table(cpu, vaddr1, memblock,
 			    wf1, paddr1);
+
+		/*  Set new last_written_tlb_index hint:  */
+		cpu->cd.mips.last_written_tlb_index = index;
 	}
 }
 
