@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_ppc.c,v 1.61 2006/07/24 21:14:52 debug Exp $
+ *  $Id: cpu_ppc.c,v 1.64 2006/09/21 11:53:26 debug Exp $
  *
  *  PowerPC/POWER CPU emulation.
  */
@@ -46,6 +46,7 @@
 #include "ppc_pte.h"
 #include "ppc_spr.h"
 #include "ppc_spr_strings.h"
+#include "settings.h"
 #include "symbol.h"
 
 #define	DYNTRANS_DUALMODE_32
@@ -86,10 +87,10 @@ int ppc_cpu_new(struct cpu *cpu, struct memory *mem, struct machine *machine,
 
 	cpu->memory_rw = ppc_memory_rw;
 
-	cpu->cd.ppc.cpu_type    = cpu_type_defs[found];
-	cpu->name               = cpu->cd.ppc.cpu_type.name;
-	cpu->byte_order         = EMUL_BIG_ENDIAN;
-	cpu->cd.ppc.mode        = MODE_PPC;	/*  TODO  */
+	cpu->cd.ppc.cpu_type = cpu_type_defs[found];
+	cpu->name            = cpu->cd.ppc.cpu_type.name;
+	cpu->byte_order      = EMUL_BIG_ENDIAN;
+	cpu->cd.ppc.mode     = MODE_PPC;	/*  TODO  */
 
 	/*  Current operating mode:  */
 	cpu->cd.ppc.bits = cpu->cd.ppc.cpu_type.bits;
@@ -168,6 +169,52 @@ int ppc_cpu_new(struct cpu *cpu, struct memory *mem, struct machine *machine,
 	if (cpu->machine->prom_emulation)
 		cpu->cd.ppc.of_emul_addr = 0xfff00000;
 
+	/*  Add all register names to the settings:  */
+	CPU_SETTINGS_ADD_REGISTER64("pc", cpu->pc);
+	CPU_SETTINGS_ADD_REGISTER64("msr", cpu->cd.ppc.msr);
+	CPU_SETTINGS_ADD_REGISTER64("ctr", cpu->cd.ppc.spr[SPR_CTR]);
+	CPU_SETTINGS_ADD_REGISTER64("xer", cpu->cd.ppc.spr[SPR_XER]);
+	CPU_SETTINGS_ADD_REGISTER64("dec", cpu->cd.ppc.spr[SPR_DEC]);
+	CPU_SETTINGS_ADD_REGISTER64("hdec", cpu->cd.ppc.spr[SPR_HDEC]);
+	CPU_SETTINGS_ADD_REGISTER64("srr0", cpu->cd.ppc.spr[SPR_SRR0]);
+	CPU_SETTINGS_ADD_REGISTER64("srr1", cpu->cd.ppc.spr[SPR_SRR1]);
+	CPU_SETTINGS_ADD_REGISTER64("sdr1", cpu->cd.ppc.spr[SPR_SDR1]);
+	CPU_SETTINGS_ADD_REGISTER64("ibat0u", cpu->cd.ppc.spr[SPR_IBAT0U]);
+	CPU_SETTINGS_ADD_REGISTER64("ibat0l", cpu->cd.ppc.spr[SPR_IBAT0L]);
+	CPU_SETTINGS_ADD_REGISTER64("ibat1u", cpu->cd.ppc.spr[SPR_IBAT1U]);
+	CPU_SETTINGS_ADD_REGISTER64("ibat1l", cpu->cd.ppc.spr[SPR_IBAT1L]);
+	CPU_SETTINGS_ADD_REGISTER64("ibat2u", cpu->cd.ppc.spr[SPR_IBAT2U]);
+	CPU_SETTINGS_ADD_REGISTER64("ibat2l", cpu->cd.ppc.spr[SPR_IBAT2L]);
+	CPU_SETTINGS_ADD_REGISTER64("ibat3u", cpu->cd.ppc.spr[SPR_IBAT3U]);
+	CPU_SETTINGS_ADD_REGISTER64("ibat3l", cpu->cd.ppc.spr[SPR_IBAT3L]);
+	CPU_SETTINGS_ADD_REGISTER64("dbat0u", cpu->cd.ppc.spr[SPR_DBAT0U]);
+	CPU_SETTINGS_ADD_REGISTER64("dbat0l", cpu->cd.ppc.spr[SPR_DBAT0L]);
+	CPU_SETTINGS_ADD_REGISTER64("dbat1u", cpu->cd.ppc.spr[SPR_DBAT1U]);
+	CPU_SETTINGS_ADD_REGISTER64("dbat1l", cpu->cd.ppc.spr[SPR_DBAT1L]);
+	CPU_SETTINGS_ADD_REGISTER64("dbat2u", cpu->cd.ppc.spr[SPR_DBAT2U]);
+	CPU_SETTINGS_ADD_REGISTER64("dbat2l", cpu->cd.ppc.spr[SPR_DBAT2L]);
+	CPU_SETTINGS_ADD_REGISTER64("dbat3u", cpu->cd.ppc.spr[SPR_DBAT3U]);
+	CPU_SETTINGS_ADD_REGISTER64("dbat3l", cpu->cd.ppc.spr[SPR_DBAT3L]);
+	CPU_SETTINGS_ADD_REGISTER64("lr", cpu->cd.ppc.spr[SPR_LR]);
+	CPU_SETTINGS_ADD_REGISTER32("cr", cpu->cd.ppc.cr);
+	CPU_SETTINGS_ADD_REGISTER32("fpscr", cpu->cd.ppc.fpscr);
+	/*  Integer GPRs, floating point registers, and segment registers:  */
+	for (i=0; i<PPC_NGPRS; i++) {
+		char tmpstr[5];
+		snprintf(tmpstr, sizeof(tmpstr), "r%i", i);
+		CPU_SETTINGS_ADD_REGISTER64(tmpstr, cpu->cd.ppc.gpr[i]);
+	}
+	for (i=0; i<PPC_NFPRS; i++) {
+		char tmpstr[5];
+		snprintf(tmpstr, sizeof(tmpstr), "f%i", i);
+		CPU_SETTINGS_ADD_REGISTER64(tmpstr, cpu->cd.ppc.fpr[i]);
+	}
+	for (i=0; i<16; i++) {
+		char tmpstr[5];
+		snprintf(tmpstr, sizeof(tmpstr), "sr%i", i);
+		CPU_SETTINGS_ADD_REGISTER32(tmpstr, cpu->cd.ppc.sr[i]);
+	}
+
 	return 1;
 }
 
@@ -480,7 +527,7 @@ void ppc_cpu_register_dump(struct cpu *cpu, int gprs, int coprocs)
 		for (i=0; i<16; i++) {
 			uint32_t s = cpu->cd.ppc.sr[i];
 			debug("cpu%i:", x);
-			debug("  sr%2i = 0x%08x", i, (int)s);
+			debug("  sr%-2i = 0x%08x", i, (int)s);
 			s &= (SR_TYPE | SR_SUKEY | SR_PRKEY | SR_NOEXEC);
 			if (s != 0) {
 				debug("  (");
@@ -512,95 +559,6 @@ void ppc_cpu_register_dump(struct cpu *cpu, int gprs, int coprocs)
 }
 
 
-/*
- *  ppc_cpu_register_match():
- */
-void ppc_cpu_register_match(struct machine *m, char *name,
-	int writeflag, uint64_t *valuep, int *match_register)
-{
-	int cpunr = 0;
-
-	/*  CPU number:  */
-
-	/*  TODO  */
-
-	/*  Register name:  */
-	if (strcasecmp(name, "pc") == 0) {
-		if (writeflag) {
-			m->cpus[cpunr]->pc = *valuep;
-		} else
-			*valuep = m->cpus[cpunr]->pc;
-		*match_register = 1;
-	} else if (strcasecmp(name, "msr") == 0) {
-		if (writeflag)
-			m->cpus[cpunr]->cd.ppc.msr = *valuep;
-		else
-			*valuep = m->cpus[cpunr]->cd.ppc.msr;
-		*match_register = 1;
-	} else if (strcasecmp(name, "lr") == 0) {
-		if (writeflag)
-			m->cpus[cpunr]->cd.ppc.spr[SPR_LR] = *valuep;
-		else
-			*valuep = m->cpus[cpunr]->cd.ppc.spr[SPR_LR];
-		*match_register = 1;
-	} else if (strcasecmp(name, "cr") == 0) {
-		if (writeflag)
-			m->cpus[cpunr]->cd.ppc.cr = *valuep;
-		else
-			*valuep = m->cpus[cpunr]->cd.ppc.cr;
-		*match_register = 1;
-	} else if (strcasecmp(name, "dec") == 0) {
-		if (writeflag)
-			m->cpus[cpunr]->cd.ppc.spr[SPR_DEC] = *valuep;
-		else
-			*valuep = m->cpus[cpunr]->cd.ppc.spr[SPR_DEC];
-		*match_register = 1;
-	} else if (strcasecmp(name, "hdec") == 0) {
-		if (writeflag)
-			m->cpus[cpunr]->cd.ppc.spr[SPR_HDEC] = *valuep;
-		else
-			*valuep = m->cpus[cpunr]->cd.ppc.spr[SPR_HDEC];
-		*match_register = 1;
-	} else if (strcasecmp(name, "ctr") == 0) {
-		if (writeflag)
-			m->cpus[cpunr]->cd.ppc.spr[SPR_CTR] = *valuep;
-		else
-			*valuep = m->cpus[cpunr]->cd.ppc.spr[SPR_CTR];
-		*match_register = 1;
-	} else if (name[0] == 'r' && isdigit((int)name[1])) {
-		int nr = atoi(name + 1);
-		if (nr >= 0 && nr < PPC_NGPRS) {
-			if (writeflag) {
-				m->cpus[cpunr]->cd.ppc.gpr[nr] = *valuep;
-			} else
-				*valuep = m->cpus[cpunr]->cd.ppc.gpr[nr];
-			*match_register = 1;
-		}
-	} else if (strcasecmp(name, "xer") == 0) {
-		if (writeflag)
-			m->cpus[cpunr]->cd.ppc.spr[SPR_XER] = *valuep;
-		else
-			*valuep = m->cpus[cpunr]->cd.ppc.spr[SPR_XER];
-		*match_register = 1;
-	} else if (strcasecmp(name, "fpscr") == 0) {
-		if (writeflag)
-			m->cpus[cpunr]->cd.ppc.fpscr = *valuep;
-		else
-			*valuep = m->cpus[cpunr]->cd.ppc.fpscr;
-		*match_register = 1;
-	} else if (name[0] == 'f' && isdigit((int)name[1])) {
-		int nr = atoi(name + 1);
-		if (nr >= 0 && nr < PPC_NFPRS) {
-			if (writeflag) {
-				m->cpus[cpunr]->cd.ppc.fpr[nr] = *valuep;
-			} else
-				*valuep = m->cpus[cpunr]->cd.ppc.fpr[nr];
-			*match_register = 1;
-		}
-	}
-}
-
-
 /*
  *  ppc_cpu_tlbdump():
  *
