@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_ppc_instr_loadstore.c,v 1.6 2005/11/23 06:59:52 debug Exp $
+ *  $Id: cpu_ppc_instr_loadstore.c,v 1.9 2006/04/02 10:21:08 debug Exp $
  *
  *  POWER/PowerPC load/store instructions.
  *
@@ -44,7 +44,11 @@
 void LS_GENERIC_N(struct cpu *cpu, struct ppc_instr_call *ic)
 {
 #ifdef MODE32
-	uint32_t addr = reg(ic->arg[1]) +
+	uint32_t addr =
+#else
+	uint64_t addr =
+#endif
+	    reg(ic->arg[1]) +
 #ifdef LS_INDEXED
 	    reg(ic->arg[2]);
 #else
@@ -161,9 +165,6 @@ void LS_GENERIC_N(struct cpu *cpu, struct ppc_instr_call *ic)
 #ifdef LS_UPDATE
 	reg(ic->arg[1]) = addr;
 #endif
-#else	/*  !MODE32  */
-	fatal("TODO: mode64\n");
-#endif	/*  !MODE32  */
 }
 #endif
 
@@ -171,7 +172,11 @@ void LS_GENERIC_N(struct cpu *cpu, struct ppc_instr_call *ic)
 void LS_N(struct cpu *cpu, struct ppc_instr_call *ic)
 {
 #ifdef MODE32
-	uint32_t addr = reg(ic->arg[1])
+	uint32_t addr =
+#else
+	uint64_t addr =
+#endif
+	    reg(ic->arg[1])
 #ifdef LS_INDEXED
 	    + reg(ic->arg[2])
 #else
@@ -199,6 +204,17 @@ void LS_N(struct cpu *cpu, struct ppc_instr_call *ic)
 	}
 #endif
 
+
+#ifndef MODE32
+/*******************************************/
+if (!cpu->is_32bit) {
+LS_GENERIC_N(cpu, ic);
+return;
+}
+/*******************************************/
+#endif
+
+
 	if (page == NULL) {
 		LS_GENERIC_N(cpu, ic);
 		return;
@@ -294,10 +310,5 @@ void LS_N(struct cpu *cpu, struct ppc_instr_call *ic)
 #ifdef LS_UPDATE
 	reg(ic->arg[1]) = new_addr;
 #endif
-
-#else	/*  !MODE32  */
-	fatal("ppc load/store mode64: TODO\n");
-	exit(1);
-#endif
 }
 
