@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_sparc.c,v 1.35 2006/07/23 12:40:24 debug Exp $
+ *  $Id: cpu_sparc.c,v 1.38 2006/09/19 10:50:08 debug Exp $
  *
  *  SPARC CPU emulation.
  */
@@ -39,6 +39,7 @@
 #include "machine.h"
 #include "memory.h"
 #include "misc.h"
+#include "settings.h"
 #include "symbol.h"
 
 
@@ -90,6 +91,9 @@ int sparc_cpu_new(struct cpu *cpu, struct memory *mem, struct machine *machine,
 
 	cpu->instruction_has_delayslot = sparc_cpu_instruction_has_delayslot;
 
+	/*  TODO: Separate this into 64-bit vs 32-bit?  */
+	cpu->translate_v2p = sparc_translate_v2p;
+
 	if (cpu->is_32bit) {
 		cpu->run_instr = sparc32_run_instr;
 		cpu->update_translation_table =
@@ -147,6 +151,14 @@ int sparc_cpu_new(struct cpu *cpu, struct memory *mem, struct machine *machine,
 		exit(1);
 	}
 
+	CPU_SETTINGS_ADD_REGISTER64("pc", cpu->pc);
+	CPU_SETTINGS_ADD_REGISTER64("y", cpu->cd.sparc.y);
+	CPU_SETTINGS_ADD_REGISTER64("pstate", cpu->cd.sparc.pstate);
+	for (i=0; i<N_SPARC_REG; i++)
+		CPU_SETTINGS_ADD_REGISTER64(sparc_regnames[i],
+		    cpu->cd.sparc.r[i]);
+	/*  TODO: Handler for writes to the zero register!  */
+
 	return 1;
 }
 
@@ -278,50 +290,6 @@ void sparc_cpu_register_dump(struct cpu *cpu, int gprs, int coprocs)
 }
 
 
-/*
- *  sparc_cpu_register_match():
- */
-void sparc_cpu_register_match(struct machine *m, char *name,
-	int writeflag, uint64_t *valuep, int *match_register)
-{
-	int i, cpunr = 0;
-
-	/*  CPU number:  */
-	/*  TODO  */
-
-	for (i=0; i<N_SPARC_REG; i++) {
-		if (strcasecmp(name, sparc_regnames[i]) == 0) {
-			if (writeflag && i != SPARC_ZEROREG)
-				m->cpus[cpunr]->cd.sparc.r[i] = *valuep;
-			else
-				*valuep = m->cpus[cpunr]->cd.sparc.r[i];
-			*match_register = 1;
-		}
-	}
-
-	if (strcasecmp(name, "pc") == 0) {
-		if (writeflag) {
-			m->cpus[cpunr]->pc = *valuep;
-		} else {
-			*valuep = m->cpus[cpunr]->pc;
-		}
-		*match_register = 1;
-	}
-
-	if (strcasecmp(name, "y") == 0) {
-		if (writeflag) {
-			m->cpus[cpunr]->cd.sparc.y = (uint32_t) *valuep;
-		} else {
-			*valuep = (uint32_t) m->cpus[cpunr]->cd.sparc.y;
-		}
-		*match_register = 1;
-	}
-
-	if (*match_register && m->cpus[cpunr]->is_32bit)
-		(*valuep) &= 0xffffffffULL;
-}
-
-
 /*
  *  sparc_cpu_tlbdump():
  *
@@ -472,6 +440,8 @@ int sparc_cpu_instruction_has_delayslot(struct cpu *cpu, unsigned char *ib)
 		switch (op2) {
 		case 56:/*  jump and link  */
 			return 1;
+		case 57:/*  return  */
+			return 1;
 		}
 		break;
 	}
