@@ -25,16 +25,32 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: memory_mips_v2p.c,v 1.9 2006/08/12 11:43:13 debug Exp $
+ *  $Id: memory_mips_v2p.c,v 1.14 2006/10/14 23:47:37 debug Exp $
  */
 
 
 /*
  *  translate_v2p():
  *
- *  Don't call this function is userland_emul is non-NULL, or cpu is NULL.
+ *  Translate a virtual MIPS address to a physical address, by looking up the
+ *  address in the TLB. On failure, an exception is generated (except for the
+ *  case when FLAG_NOEXCEPTIONS is used).
  *
- *  TODO:  vpn2 is a bad name for R2K/R3K, as it is the actual framenumber.
+ *  Note: This function is long and hairy, it is included several times with
+ *        various defines set, to produce more or less optimized versions of
+ *        the function for different emulated CPU types.
+ *
+ *  V2P_MMU3K       Defined for R2000/R3000 emulation. If it is not defined,
+ *                  R4000+/MIPS32/MIPS64 is presumed.
+ *  V2P_MMU10K      This enables the use of 44 userspace bits, instead of 40.
+ *  V2P_MMU4100     VR41xx processors support 1 KB pages, so their page mask
+ *                  is slightly different. (The emulator only supports 4 KB
+ *                  pages, though.)
+ *  V2P_MMU8K       Not yet. (TODO.)
+ *
+ *
+ *  Note:  Unfortunately, the variable name vpn2 is poorly choosen for R2K/R3K,
+ *         since it actual contains the vpn.
  *
  *  Return values:
  *	0  Failure
@@ -55,16 +71,18 @@ int TRANSLATE_ADDRESS(struct cpu *cpu, uint64_t vaddr,
 	const int x_64 = 0;
 	const int n_tlbs = 64;
 	const int pmask = 0xfff;
+	uint64_t xuseg_top;		/*  Well, useg actually.  */
 #else
 #ifdef V2P_MMU10K
-	const uint64_t vpn2_mask = ENTRYHI_VPN2_MASK_R10K;
+	const uint64_t vpn2_mask = ENTRYHI_R_MASK | ENTRYHI_VPN2_MASK_R10K;
+	uint64_t xuseg_top = ENTRYHI_VPN2_MASK_R10K | 0x1fffULL;
 #else
 #ifdef V2P_MMU4100
-/* This is ugly  */
-	const uint64_t vpn2_mask = ENTRYHI_VPN2_MASK | 0x1800;
+	const uint64_t vpn2_mask = ENTRYHI_R_MASK | ENTRYHI_VPN2_MASK | 0x1800;
 #else
-	const uint64_t vpn2_mask = ENTRYHI_VPN2_MASK;
+	const uint64_t vpn2_mask = ENTRYHI_R_MASK | ENTRYHI_VPN2_MASK;
 #endif
+	uint64_t xuseg_top = ENTRYHI_VPN2_MASK | 0x1fffULL;
 #endif
 	int x_64;	/*  non-zero for 64-bit address space accesses  */
 	int pageshift, n_tlbs;
@@ -89,11 +107,11 @@ int TRANSLATE_ADDRESS(struct cpu *cpu, uint64_t vaddr,
 	status = cp0->reg[COP0_STATUS];
 
 	/*
-	 *  R4000 Address Translation:
+	 *  MIPS R4000+ and MIPS64 Address Translation:
 	 *
-	 *  An address may be in one of the kernel segments, that
-	 *  are directly mapped, or the address can go through the
-	 *  TLBs to be turned into a physical address.
+	 *  An address may be in one of the kernel segments, that are directly
+	 *  mapped to physical addresses, or the address needs to be looked up
+	 *  in the TLB entries.
 	 *
 	 *  KSU: EXL: ERL: X:  Name:   Range:
 	 *  ---- ---- ---- --  -----   ------
@@ -114,25 +132,18 @@ int TRANSLATE_ADDRESS(struct cpu *cpu, uint64_t vaddr,
 	 *					  unmapped, cached
 	 *   00   x    x    0  kseg1   0xa0000000 - 0xbfffffff (0.5GB)
 	 *					  unmapped, uncached
-	 *   00   x    x    0  ksseg   0xc0000000 - 0xdfffffff (0.5GB)
-	 *					  (via TLB)
-	 *   00   x    x    0  kseg3   0xe0000000 - 0xffffffff (0.5GB)
-	 *					  (via TLB)
+	 *   00   x    x    0  ksseg   0xc0000000 - 0xdfffffff (0.5GB) (via TLB)
+	 *   00   x    x    0  kseg3   0xe0000000 - 0xffffffff (0.5GB) (via TLB)
 	 *   00   x    x    1  xksuseg 0 - 0xffffffffff (1TB) (via TLB) (*)
 	 *   00   x    x    1  xksseg  0x4000000000000000 - 0x400000ffffffffff
 	 *					  (1TB)  (via TLB)
 	 *   00   x    x    1  xkphys  0x8000000000000000 - 0xbfffffffffffffff
-	 *					  todo
 	 *   00   x    x    1  xkseg   0xc000000000000000 - 0xc00000ff7fffffff
-	 *					  todo
 	 *   00   x    x    1  ckseg0  0xffffffff80000000 - 0xffffffff9fffffff
-	 *					  like kseg0
 	 *   00   x    x    1  ckseg1  0xffffffffa0000000 - 0xffffffffbfffffff
-	 *					  like kseg1
 	 *   00   x    x    1  cksseg  0xffffffffc0000000 - 0xffffffffdfffffff
-	 *					  like ksseg
 	 *   00   x    x    1  ckseg3  0xffffffffe0000000 - 0xffffffffffffffff
-	 *					  like kseg2
+	 *					  like 0x80000000 - 0xffffffff
 	 *
 	 *  (*) = if ERL=1 then kuseg is not via TLB, but unmapped,
 	 *  uncached physical memory.
@@ -154,45 +165,51 @@ int TRANSLATE_ADDRESS(struct cpu *cpu, uint64_t vaddr,
 	vaddr_asid = cp0->reg[COP0_ENTRYHI] & R2K3K_ENTRYHI_ASID_MASK;
 	vaddr_vpn2 = vaddr & R2K3K_ENTRYHI_VPN_MASK;
 #else
-	/*
-	 *  R4000 and others:
-	 *
-	 *  kx,sx,ux = 0 for 32-bit addressing,
-	 *  1 for 64-bit addressing. 
-	 */
-	n_tlbs = cpu->cd.mips.cpu_type.nr_of_tlb_entries;
-
+	/*  kx,sx,ux = 0 for 32-bit addressing, 1 for 64-bit addressing.  */
 	ksu = (status & STATUS_KSU_MASK) >> STATUS_KSU_SHIFT;
 	if (status & (STATUS_EXL | STATUS_ERL))
 		ksu = KSU_KERNEL;
 
-	/*  Assume KSU_USER.  */
-	x_64 = status & STATUS_UX;
-
-	if (ksu == KSU_KERNEL)
+	switch (ksu) {
+	case KSU_USER:
+		x_64 = status & STATUS_UX;
+		break;
+	case KSU_KERNEL:
 		x_64 = status & STATUS_KX;
-	else if (ksu == KSU_SUPERVISOR)
+		break;
+	case KSU_SUPERVISOR:
 		x_64 = status & STATUS_SX;
+		/*  FALLTHROUGH, since supervisor address spaces are not
+		    really implemented yet.  */
+	default:fatal("memory_mips_v2p.c: ksu=%i not yet implemented yet\n",
+		    ksu);
+		exit(1);
+	}
 
-	/*  This suppresses a compiler warning:  */
+	n_tlbs = cpu->cd.mips.cpu_type.nr_of_tlb_entries;
+
+	/*  Having this here suppresses a compiler warning:  */
 	pageshift = 12;
 
+	/*  KUSEG: 0x00000000 - 0x7fffffff if ERL = 1 and KSU = kernel:  */
+	if (ksu == KSU_KERNEL && (status & STATUS_ERL) &&
+	    vaddr <= 0x7fffffff) {
+		*return_paddr = vaddr & 0x7fffffff;
+		return 2;
+	}
+
 	/*
-	 *  Physical addressing on R10000 etc:
+	 *  XKPHYS: 0x8000000000000000 - 0xbfffffffffffffff
 	 *
-	 *  TODO: Probably only accessible in kernel mode.
+	 *  TODO: Is the correct error generated if accessing XKPHYS from
+	 *        usermode?
 	 *
-	 *  0x9000000080000000 = disable L2 cache (?)
-	 *  TODO:  Make this correct.
+	 *  TODO: Magic on SGI machines... Cache control, NUMA, etc.:
+	 *        0x9000000080000000   = disable L2 cache (?)
+	 *        0x90000000a0000000   = something on IP30?
+	 *        0x92.... and 0x96... = NUMA on IP27
 	 */
-	if ((vaddr >> 62) == 0x2) {
-		/*
-		 *  On IP30, addresses such as 0x900000001f600050 are used,
-		 *  but also things like 0x90000000a0000000.  (TODO)
-		 *
-		 *  On IP27 (and probably others), addresses such as
-		 *  0x92... and 0x96... have to do with NUMA stuff.
-		 */
+	if (ksu == KSU_KERNEL && (vaddr & ENTRYHI_R_MASK) == ENTRYHI_R_XKPHYS) {
 		*return_paddr = vaddr & (((uint64_t)1 << 44) - 1);
 		return 2;
 	}
@@ -202,17 +219,16 @@ int TRANSLATE_ADDRESS(struct cpu *cpu, uint64_t vaddr,
 	/*  vpn2 depends on pagemask, which is not fixed on R4000  */
 #endif
 
+	/*  If 32-bit, truncate address and sign extend:  */
+	if (x_64 == 0) {
+		vaddr = (int32_t) vaddr;
+		xuseg_top = 0x7fffffff;
+		/*  (Actually useg for R2000/R3000)  */
+	}
 
-	if (vaddr <= 0x7fffffff)
+	if (vaddr <= xuseg_top) {
 		use_tlb = 1;
-	else {
-#if 1
-/*  TODO: This should be removed, but it seems that other
-bugs are triggered.  */
-		/*  Sign-extend vaddr, if necessary:  */
-		if ((vaddr >> 32) == 0 && vaddr & (uint32_t)0x80000000ULL)
-			vaddr |= 0xffffffff00000000ULL;
-#endif
+	} else {
 		if (ksu == KSU_KERNEL) {
 			/*  kseg0, kseg1:  */
 			if (vaddr >= (uint64_t)0xffffffff80000000ULL &&
@@ -221,12 +237,11 @@ bugs are triggered.  */
 				return 2;
 			}
 
-			/*  TODO: supervisor stuff  */
-
 			/*  other segments:  */
 			use_tlb = 1;
-		} else
+		} else {
 			use_tlb = 0;
+		}
 	}
 
 	if (use_tlb) {
@@ -237,8 +252,13 @@ bugs are triggered.  */
 		int g_bit, v_bit, d_bit;
 		uint64_t cached_hi, cached_lo0;
 		uint64_t entry_vpn2 = 0, entry_asid, pfn;
+		int i_end;
 
-		for (i=0; i<n_tlbs; i++) {
+		i = cpu->cd.mips.last_written_tlb_index;
+		i_end = i == 0? n_tlbs-1 : i - 1;
+
+		/*  Scan all TLB entries:  */
+		for (;;) {
 #ifdef V2P_MMU3K
 			/*  R3000 or similar:  */
 			cached_hi = cp0->tlbs[i].hi;
@@ -381,6 +401,14 @@ bugs are triggered.  */
 					goto exception;
 				}
 			}
+
+			if (i == i_end)
+				break;
+
+			/*  Go to the next TLB entry:  */
+			i ++;
+			if (i == n_tlbs)
+				i = 0;
 		}
 	}
 
