@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: debugger.c,v 1.14 2006/07/01 21:15:46 debug Exp $
+ *  $Id: debugger.c,v 1.20 2006/10/29 05:17:21 debug Exp $
  *
  *  Single-step debugger.
  *
@@ -38,12 +38,6 @@
  *
  *	Call stack display (back-trace)?
  *
- *	More generic expression evaluator (for example + - * / between multiple
- *	terms), including _TAB COMPLETION_ of symbols and register names!
- *	Must be possible to reach any emul, any machine in any emul, any
- *	cpu in any machine, other variables in the emulator, and so forth.
- *	Perhaps a "settable variables registry" somewhere?
- *
  *	Nicer looking output of register dumps, floating point registers,
  *	etc. Warn about weird/invalid register contents.
  *
@@ -71,6 +65,7 @@
 #include "misc.h"
 #include "net.h"
 #include "settings.h"
+#include "timer.h"
 #include "x11.h"
 
 
@@ -106,8 +101,13 @@ static volatile int ctrl_c;
 
 static int debugger_n_emuls;
 static struct emul **debugger_emuls;
-static struct emul *debugger_emul;
+
+/*  Currently focused CPU, machine, and emulation:  */
+int debugger_cur_cpu;
+int debugger_cur_machine;
+int debugger_cur_emul;
 static struct machine *debugger_machine;
+static struct emul *debugger_emul;
 
 #define	MAX_CMD_BUFLEN		72
 #define	N_PREVIOUS_CMDS		150
@@ -150,7 +150,7 @@ char debugger_readchar(void)
 			be factored out...  */
 
 		/*  Give up some CPU time:  */
-		usleep(1);
+		usleep(10000);
 	}
 	return ch;
 }
@@ -193,131 +193,6 @@ void debugger_activate(int x)
 }
 
 
-/*
- *  debugger_parse_name():
- *
- *  This function reads a string, and tries to match it to a register name,
- *  a symbol, or treat it as a decimal numeric value.
- *
- *  Some examples:
- *
- *	"0x7fff1234"		==> numeric value (hex, in this case)
- *	"pc", "r5", "hi", "t4"	==> register (CPU dependent)
- *	"memcpy+64"		==> symbol (plus offset)
- *
- *  Register names can be preceeded by "x:" where x is the CPU number. (CPU
- *  0 is assumed by default.)
- *
- *  To force detection of different types, a character can be added in front of
- *  the name: "$" for numeric values, "%" for registers, and "@" for symbols.
- *
- *  Return value is:
- *
- *	NAME_PARSE_NOMATCH	no match
- *	NAME_PARSE_MULTIPLE	multiple matches
- *
- *  or one of these (and then *valuep is read or written, depending on
- *  the writeflag):
- *
- *	NAME_PARSE_REGISTER	a register
- *	NAME_PARSE_NUMBER	a hex number
- *	NAME_PARSE_SYMBOL	a symbol
- */
-#define	NAME_PARSE_NOMATCH	0
-#define	NAME_PARSE_MULTIPLE	1
-#define	NAME_PARSE_REGISTER	2
-#define	NAME_PARSE_NUMBER	3
-#define	NAME_PARSE_SYMBOL	4
-static int debugger_parse_name(struct machine *m, char *name, int writeflag,
-	uint64_t *valuep)
-{
-	int match_register = 0, match_symbol = 0, match_numeric = 0;
-	int skip_register, skip_numeric, skip_symbol;
-
-	if (m == NULL || name == NULL) {
-		fprintf(stderr, "debugger_parse_name(): NULL ptr\n");
-		exit(1);
-	}
-
-	/*  Warn about non-signextended values:  */
-	if (writeflag) {
-		if (m->cpus[0]->is_32bit) {
-			/*  Automagically sign-extend.  TODO: Is this good?  */
-			if (((*valuep) >> 32) == 0 && (*valuep) & 0x80000000ULL)
-				(*valuep) |= 0xffffffff00000000ULL;
-		} else {
-			if (((*valuep) >> 32) == 0 && (*valuep) & 0x80000000ULL)
-				printf("WARNING: The value is not sign-extende"
-				    "d. Is this what you intended?\n");
-		}
-	}
-
-	skip_register = name[0] == '$' || name[0] == '@';
-	skip_numeric  = name[0] == '%' || name[0] == '@';
-	skip_symbol   = name[0] == '$' || name[0] == '%';
-
-	/*  Check for a register match:  */
-	if (!skip_register && strlen(name) >= 1)
-		cpu_register_match(m, name, writeflag, valuep,
-		    &match_register);
-
-	/*  Check for a number match:  */
-	if (!skip_numeric && isdigit((int)name[0])) {
-		uint64_t x;
-		x = strtoull(name, NULL, 0);
-		if (writeflag) {
-			printf("You cannot assign like that.\n");
-		} else
-			*valuep = x;
-		match_numeric = 1;
-	}
-
-	/*  Check for a symbol match:  */
-	if (!skip_symbol) {
-		int res;
-		char *p, *sn;
-		uint64_t newaddr, ofs = 0;
-
-		sn = malloc(strlen(name) + 1);
-		if (sn == NULL) {
-			fprintf(stderr, "out of memory in debugger\n");
-			exit(1);
-		}
-		strlcpy(sn, name, strlen(name)+1);
-
-		/*  Is there a '+' in there? Then treat that as an offset:  */
-		p = strchr(sn, '+');
-		if (p != NULL) {
-			*p = '\0';
-			ofs = strtoull(p+1, NULL, 0);
-		}
-
-		res = get_symbol_addr(&m->symbol_context, sn, &newaddr);
-		if (res) {
-			if (writeflag) {
-				printf("You cannot assign like that.\n");
-			} else
-				*valuep = newaddr + ofs;
-			match_symbol = 1;
-		}
-
-		free(sn);
-	}
-
-	if (match_register + match_symbol + match_numeric > 1)
-		return NAME_PARSE_MULTIPLE;
-
-	if (match_register)
-		return NAME_PARSE_REGISTER;
-	if (match_numeric)
-		return NAME_PARSE_NUMBER;
-	if (match_symbol)
-		return NAME_PARSE_SYMBOL;
-
-	return NAME_PARSE_NOMATCH;
-}
-
-
 /*
  *  show_breakpoint():
  */
@@ -381,23 +256,23 @@ void debugger_assignment(struct machine *m, char *cmd)
 
 	/*  printf("left  = '%s'\nright = '%s'\n", left, right);  */
 
-	res_right = debugger_parse_name(m, right, 0, &tmp);
+	res_right = debugger_parse_expression(m, right, 0, &tmp);
 	switch (res_right) {
-	case NAME_PARSE_NOMATCH:
+	case PARSE_NOMATCH:
 		printf("No match for the right-hand side of the assignment.\n");
 		break;
-	case NAME_PARSE_MULTIPLE:
+	case PARSE_MULTIPLE:
 		printf("Multiple matches for the right-hand side of the "
 		    "assignment.\n");
 		break;
 	default:
-		res_left = debugger_parse_name(m, left, 1, &tmp);
+		res_left = debugger_parse_expression(m, left, 1, &tmp);
 		switch (res_left) {
-		case NAME_PARSE_NOMATCH:
+		case PARSE_NOMATCH:
 			printf("No match for the left-hand side of the "
 			    "assignment.\n");
 			break;
-		case NAME_PARSE_MULTIPLE:
+		case PARSE_MULTIPLE:
 			printf("Multiple matches for the left-hand side "
 			    "of the assignment.\n");
 			break;
@@ -710,11 +585,11 @@ static char *debugger_readline(void)
 		} else if (ch == 27) {
 			/*  Escape codes: (cursor keys etc)  */
 			while ((ch = console_readchar(MAIN_CONSOLE)) < 0)
-				usleep(1);
+				usleep(10000);
 			if (ch == '[' || ch == 'O') {
 				while ((ch = console_readchar(MAIN_CONSOLE))
 				    < 0)
-					usleep(1);
+					usleep(10000);
 				switch (ch) {
 				case '2':	/*  2~ = ins  */
 				case '5':	/*  5~ = pgup  */
@@ -722,14 +597,14 @@ static char *debugger_readline(void)
 					/*  TODO: Ugly hack, but might work.  */
 					while ((ch = console_readchar(
 					    MAIN_CONSOLE)) < 0)
-						usleep(1);
+						usleep(10000);
 					/*  Do nothing for these keys.  */
 					break;
 				case '3':	/*  3~ = delete  */
 					/*  TODO: Ugly hack, but might work.  */
 					while ((ch = console_readchar(
 					    MAIN_CONSOLE)) < 0)
-						usleep(1);
+						usleep(10000);
 					console_makeavail(MAIN_CONSOLE, '\b');
 					break;
 				case 'A':	/*  Up.  */
@@ -818,6 +693,9 @@ void debugger(void)
 	}
 
 
+	/*  Stop timers while interacting with the user:  */
+	timer_stop();
+
 	exit_debugger = 0;
 
 	while (!exit_debugger) {
@@ -858,6 +736,11 @@ void debugger(void)
 			return;
 	}
 
+	/*  Start up timers again:  */
+	timer_start();
+
+	/*  ... and reset starttime, so that nr of instructions per second
+	    can be calculated correctly:  */
 	gettimeofday(&debugger_machine->starttime, NULL);
 	debugger_machine->ninstrs_since_gettimeofday = 0;
 
@@ -912,6 +795,10 @@ void debugger_init(struct emul **emuls, int n_emuls)
 
 	debugger_machine = emuls[0]->machines[0];
 
+	debugger_cur_cpu = 0;
+	debugger_cur_machine = 0;
+	debugger_cur_emul = 0;
+
 	for (i=0; i<N_PREVIOUS_CMDS; i++) {
 		last_cmd[i] = malloc(MAX_CMD_BUFLEN);
 		if (last_cmd[i] == NULL) {
