@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_bebox.c,v 1.7 2005/11/16 21:15:18 debug Exp $
+ *  $Id: dev_bebox.c,v 1.9 2006/02/09 20:02:59 debug Exp $
  *
  *  Emulation of BeBox motherboard registers. See the following URL for more
  *  information:
@@ -65,9 +65,7 @@ static void check_cpu_masks(struct cpu *cpu, struct bebox_data *d)
 /*
  *  dev_bebox_access():
  */
-int dev_bebox_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len, int writeflag,
-	void *extra)
+DEVICE_ACCESS(bebox)
 {
 	struct bebox_data *d = extra;
 	uint64_t idata = 0, odata = 0;
@@ -143,10 +141,7 @@ int dev_bebox_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_bebox():
- */
-int devinit_bebox(struct devinit *devinit)
+DEVINIT(bebox)
 {
 	struct bebox_data *d;
 
