@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_bt431.c,v 1.10 2005/11/13 00:14:08 debug Exp $
+ *  $Id: dev_bt431.c,v 1.11 2006/01/01 13:17:16 debug Exp $
  *  
  *  Brooktree 431, used by TURBOchannel graphics cards.
  *
@@ -63,9 +63,7 @@ struct bt431_data {
 /*
  *  dev_bt431_access():
  */
-int dev_bt431_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(bt431)
 {
 	struct bt431_data *d = (struct bt431_data *) extra;
 	uint64_t idata = 0, odata = 0;
