@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_colorplanemask.c,v 1.11 2005/11/13 00:14:08 debug Exp $
+ *  $Id: dev_colorplanemask.c,v 1.12 2006/01/01 13:17:16 debug Exp $
  *  
  *  Color plane mask used by DECstation 3100.
  *
@@ -50,9 +50,7 @@ struct colorplanemask_data {
 /*
  *  dev_colorplanemask_access():
  */
-int dev_colorplanemask_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(colorplanemask)
 {
 	struct colorplanemask_data *d = (struct colorplanemask_data *) extra;
 
