@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_dec_ioasic.c,v 1.14 2005/11/13 00:14:08 debug Exp $
+ *  $Id: dev_dec_ioasic.c,v 1.15 2006/01/01 13:17:16 debug Exp $
  *  
  *  DECstation "3MIN" and "3MAX" IOASIC device.
  *
@@ -50,9 +50,7 @@
 /*
  *  dev_dec_ioasic_access():
  */
-int dev_dec_ioasic_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(dec_ioasic)
 {
 	struct dec_ioasic_data *d = (struct dec_ioasic_data *) extra;
 	uint64_t idata = 0, odata = 0;
