@@ -25,14 +25,13 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_fb.c,v 1.123 2006/07/24 08:08:39 debug Exp $
+ *  $Id: dev_fb.c,v 1.127 2006/10/22 04:20:53 debug Exp $
  *  
  *  Generic framebuffer device.
  *
  *	DECstation VFB01 monochrome framebuffer, 1024x864
  *	DECstation VFB02 8-bit color framebuffer, 1024x864
  *	DECstation Maxine, 1024x768 8-bit color
- *	HPC (mips, arm, ..) framebuffer
  *	Playstation 2 (24-bit color)
  *	Generic (any resolution, several bit depths possible, useful for
  *		testmachines)
@@ -219,10 +218,6 @@ void dev_fb_setcursor(struct vfb_data *d, int cursor_x, int cursor_y, int on,
 	}
 #endif
 
-	if (d->fb_window != NULL)
-		console_set_framebuffer_mouse(cursor_x, cursor_y,
-		    d->fb_window->fb_number);
-
 	/*  debug("dev_fb_setcursor(%i,%i, size %i,%i, on=%i)\n",
 	    cursor_x, cursor_y, cursor_xsize, cursor_ysize, on);  */
 }
@@ -413,11 +408,7 @@ void (*redraw[2 * 4 * 2])(struct vfb_data *, int, int) = {
 #endif	/*  WITH_X11  */
 
 
-/*
- *  dev_fb_tick():
- *
- */
-void dev_fb_tick(struct cpu *cpu, void *extra)
+DEVICE_TICK(fb)
 {
 	struct vfb_data *d = extra;
 #ifdef WITH_X11
@@ -617,9 +608,6 @@ void dev_fb_tick(struct cpu *cpu, void *extra)
 }
 
 
-/*
- *  dev_fb_access():
- */
 DEVICE_ACCESS(fb)
 {
 	struct vfb_data *d = extra;
@@ -744,7 +732,8 @@ DEVICE_ACCESS(fb)
  *
  *  VFB_DEC_VFB01, _VFB02, and VFB_DEC_MAXINE are DECstation specific.
  *
- *  If type is VFB_HPC, then color encoding differs from the generic case.
+ *  VFB_HPC is like generic, but the color encoding is done as on HPCmips
+ *  and Dreamcast.
  *
  *  If bit_depth = -15 (note the minus sign), then a special hack is used for
  *  the Playstation Portable's 5-bit R, 5-bit G, 5-bit B.
