@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_kn210.c,v 1.13 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_kn210.c,v 1.15 2006/02/09 20:02:59 debug Exp $
  *  
  *  DECsystem 5400 (KN210) stuff
  */
@@ -50,9 +50,7 @@ struct kn210_data {
 /*
  *  dev_kn210_access():
  */
-int dev_kn210_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(kn210)
 {
 	uint64_t idata = 0, odata = 0;
 
@@ -77,10 +75,7 @@ int dev_kn210_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_kn210():
- */
-int devinit_kn210(struct devinit *devinit)
+DEVINIT(kn210)
 {
 	struct kn210_data *d;
 
