@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ohci.c,v 1.6 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_ohci.c,v 1.8 2006/02/09 20:02:59 debug Exp $
  *  
  *  USB OHCI (Open Host Controller Interface).
  *
@@ -62,9 +62,7 @@ struct ohci_data {
 /*
  *  dev_ohci_access():
  */
-int dev_ohci_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(ohci)
 {
 	struct ohci_data *d = extra;
 	uint64_t idata = 0, odata = 0;
@@ -151,10 +149,7 @@ fatal("URK\n");
 }
 
 
-/*
- *  devinit_ohci():
- */
-int devinit_ohci(struct devinit *devinit)
+DEVINIT(ohci)
 {
 	struct ohci_data *d;
 
