@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_pmagja.c,v 1.17 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_pmagja.c,v 1.19 2006/01/01 13:17:16 debug Exp $
  *  
  *  TURBOchannel PMAG-JA graphics device.
  *
@@ -67,13 +67,11 @@ struct pmagja_data {
 /*
  *  dev_pmagja_access():
  */
-int dev_pmagja_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(pmagja)
 {
 	uint64_t idata = 0, odata = 0;
 	struct pmagja_data *d = extra;
-	int i, res = 1;
+	size_t i, res = 1;
 
 	if (writeflag == MEM_WRITE)
 		idata = memory_readmax64(cpu, data, len);
@@ -82,7 +80,7 @@ int dev_pmagja_access(struct cpu *cpu, struct memory *mem,
 
 #ifdef JA_DEBUG
 {
-int i;
+size_t i;
 for (i=0; i<len; i++)
   if (data[i] != 0 && data[i] != 1 && data[i]!=0xff)
 	if (writeflag)
