@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_prep.c,v 1.1 2005/11/21 09:17:26 debug Exp $
+ *  $Id: dev_prep.c,v 1.4 2006/02/09 20:02:59 debug Exp $
  *
  *  PReP interrupt controller.
  */
@@ -45,9 +45,7 @@
 /*
  *  dev_prep_access():
  */
-int dev_prep_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len, int writeflag,
-	void *extra)
+DEVICE_ACCESS(prep)
 {
 	/*  struct prep_data *d = extra;  */
 	uint64_t idata = 0, odata = 0;
@@ -55,10 +53,11 @@ int dev_prep_access(struct cpu *cpu, struct memory *mem,
 	if (writeflag == MEM_WRITE)
 		idata = memory_readmax64(cpu, data, len);
 
-	if (writeflag == MEM_READ)
+	if (writeflag == MEM_READ) {
 		odata = cpu->machine->isa_pic_data.last_int;
-	else
+	} else {
 		fatal("[ prep: write to interrupt register? ]\n");
+	}
 
 	if (writeflag == MEM_READ)
 		memory_writemax64(cpu, data, len, odata);
@@ -67,10 +66,7 @@ int dev_prep_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_prep():
- */
-int devinit_prep(struct devinit *devinit)
+DEVINIT(prep)
 {
 	struct prep_data *d;
 
@@ -82,7 +78,7 @@ int devinit_prep(struct devinit *devinit)
 	memset(d, 0, sizeof(struct prep_data));
 
 	memory_device_register(devinit->machine->memory, devinit->name,
-	    0xbffffff0, 1, dev_prep_access, d, DM_DEFAULT, NULL);
+	    0xbffff000, 0x1000, dev_prep_access, d, DM_DEFAULT, NULL);
 
 	devinit->return_ptr = d;
 
