@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ps2_spd.c,v 1.13 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_ps2_spd.c,v 1.14 2006/01/01 13:17:17 debug Exp $
  *  
  *  Playstation 2 "SPD" harddisk controller.
  *
@@ -51,9 +51,7 @@ struct ps2_spd_data {
 /*
  *  dev_ps2_spd_access():
  */
-int dev_ps2_spd_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(ps2_spd)
 {
 	struct ps2_spd_data *d = extra;
 	uint64_t idata = 0, odata = 0;
