@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sgi_ip20.c,v 1.12 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_sgi_ip20.c,v 1.13 2006/01/01 13:17:17 debug Exp $
  *  
  *  SGI IP20 stuff.
  */
@@ -45,9 +45,7 @@ extern int quiet_mode;
 /*
  *  dev_sgi_ip20_access():
  */
-int dev_sgi_ip20_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(sgi_ip20)
 {
 	/*  struct sgi_ip20_data *d = (struct sgi_ip20_data *) extra;  */
 	uint64_t idata = 0, odata = 0;
