@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: diskimage.c,v 1.111 2006/08/14 17:27:45 debug Exp $
+ *  $Id: diskimage.c,v 1.114 2006/09/07 11:44:01 debug Exp $
  *
  *  Disk image support.
  *
@@ -1143,7 +1143,7 @@ xferp->data_in[4] = 0x2c - 4;	/*  Additional length  */
 
 		/*
 		 *  Bits 2..0 of buf[1] contain the 'code' which describes how
-		 *  we should space, and buf[2..4] contain the number of
+		 *  spacing should be done, and buf[2..4] contain the number of
 		 *  operations.
 		 */
 		debug("[ SPACE: buf[] = %02x %02x %02x %02x %02x %02x ]\n",
@@ -1252,6 +1252,34 @@ xferp->data_in[4] = 0x2c - 4;	/*  Additional length  */
 		diskimage__return_default_status_and_message(xferp);
 		break;
 
+	case SCSICDROM_READ_DISCINFO:
+		debug("(SCSICDROM_READ_DISCINFO: ");
+		debug("TODO");
+		retlen = 0;
+
+		/*  Return data:  */
+		scsi_transfer_allocbuf(&xferp->data_in_len,
+		    &xferp->data_in, retlen, 1);
+
+		/*  TODO  */
+
+		diskimage__return_default_status_and_message(xferp);
+		break;
+
+	case SCSICDROM_READ_TRACKINFO:
+		debug("(SCSICDROM_READ_TRACKINFO: ");
+		debug("TODO");
+		retlen = 0;
+
+		/*  Return data:  */
+		scsi_transfer_allocbuf(&xferp->data_in_len,
+		    &xferp->data_in, retlen, 1);
+
+		/*  TODO  */
+
+		diskimage__return_default_status_and_message(xferp);
+		break;
+
 	case SCSICMD_MODE_SELECT:
 		debug("[ SCSI MODE_SELECT: ");
 
