@@ -28,11 +28,13 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: debugger.h,v 1.5 2006/06/24 19:52:28 debug Exp $
+ *  $Id: debugger.h,v 1.8 2006/10/14 02:30:12 debug Exp $
  *
  *  See src/debugger/debugger.c.
  */
 
+#include "misc.h"
+
 struct emul;
 struct machine;
 
@@ -48,4 +50,14 @@ void debugger_init(struct emul **emuls, int n_emuls);
 #define	ENTER_SINGLE_STEPPING		1
 #define	SINGLE_STEPPING			2
 
+/*  debugger_expr.c:  */
+#define	PARSE_NOMATCH		0
+#define	PARSE_MULTIPLE		1
+#define	PARSE_SETTINGS		2
+#define	PARSE_NUMBER		3
+#define	PARSE_SYMBOL		4
+
+int debugger_parse_expression(struct machine *m, char *expr, int writeflag,
+	uint64_t *valuep);
+
 #endif	/*  DEBUGGER_H  */
