@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: devices.h,v 1.213 2006/07/08 12:30:03 debug Exp $
+ *  $Id: devices.h,v 1.218 2006/10/02 09:26:53 debug Exp $
  *
  *  Memory mapped devices.
  *
@@ -43,6 +43,7 @@ struct cpu;
 struct machine;
 struct memory;
 struct pci_data;
+struct timer;
 
 /* #ifdef WITH_X11
 #include <X11/Xlib.h>
@@ -279,8 +280,9 @@ struct footbridge_data {
 	uint32_t	timer_load[N_FOOTBRIDGE_TIMERS];
 	uint32_t	timer_value[N_FOOTBRIDGE_TIMERS];
 	uint32_t	timer_control[N_FOOTBRIDGE_TIMERS];
-	int		timer_being_read;
-	int		timer_poll_mode;
+
+	struct timer	*timer[N_FOOTBRIDGE_TIMERS];
+	int		pending_timer_interrupts[N_FOOTBRIDGE_TIMERS];
 
 	uint32_t        irq_status;
 	uint32_t        irq_enable;
@@ -687,6 +689,9 @@ struct vr41xx_data {
 	int		dont_clear_next;
 	int		escape_state;
 
+	int		pending_timer_interrupts;
+	struct timer	*timer;
+
 	/*  See icureg.h in NetBSD for more info.  */
 	uint16_t	sysint1;
 	uint16_t	msysint1;
@@ -722,14 +727,8 @@ struct lk201_data {
         int                     mouse_mode;
         int                     mouse_revision;         /*  0..15  */  
         int                     mouse_x, mouse_y, mouse_buttons;
-
-	int			old_host_mouse_x;
-	int			old_host_mouse_y;
-	int			old_host_mouse_stays_put;
-	int			mouse_check_interval;
-	int			mouse_check_interval_reset;
 };
-void lk201_tick(struct lk201_data *); 
+void lk201_tick(struct machine *, struct lk201_data *); 
 void lk201_tx_data(struct lk201_data *, int port, int idata);
 void lk201_init(struct lk201_data *d, int use_fb,
 	void (*add_to_rx_queue)(void *,int,int), int console_handle, void *);
