@@ -2,7 +2,7 @@
 #define	DISKIMAGE_H
 
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: diskimage.h,v 1.30 2005/11/13 02:22:01 debug Exp $
+ *  $Id: diskimage.h,v 1.32 2006/09/05 06:13:27 debug Exp $
  *
  *  Generic disk image functions.  (See diskimage.c for more info.)
  */
@@ -150,6 +150,8 @@ void diskimage_dump_info(struct machine *machine);
 /*  SCSI CD-ROM commands:  */
 #define	SCSICDROM_READ_SUBCHANNEL	0x42
 #define	SCSICDROM_READ_TOC		0x43
+#define	SCSICDROM_READ_DISCINFO		0x51
+#define	SCSICDROM_READ_TRACKINFO	0x52
 
 /*  SCSI tape commands:  */
 #define	SCSICMD_REWIND			0x01
