@@ -2,7 +2,7 @@
 #define	OPCODES_PPC_H
 
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: opcodes_ppc.h,v 1.45 2005/12/04 02:40:04 debug Exp $
+ *  $Id: opcodes_ppc.h,v 1.49 2006/03/05 16:51:55 debug Exp $
  *
  *
  *  PPC and POWER opcodes. These are gathered from various sources.
@@ -76,6 +76,7 @@
 #define	PPC_HI6_30		0x1e
 #define	  PPC_30_RLDICL		  0x0
 #define	  PPC_30_RLDICR		  0x1
+#define	  PPC_30_RLDIMI		  0x3
 #define	PPC_HI6_31		0x1f
 #define	  PPC_31_CMP		  0
 #define	  PPC_31_TW		  4
@@ -87,6 +88,7 @@
 #define	  PPC_31_LWZX		  23
 #define	  PPC_31_SLW		  24
 #define	  PPC_31_CNTLZW		  26
+#define	  PPC_31_SLD		  27
 #define	  PPC_31_AND		  28
 #define	  PPC_31_CMPL		  32
 #define	  PPC_31_SUBF		  40
@@ -112,6 +114,7 @@
 #define	  PPC_31_STWCX_DOT	  150
 #define	  PPC_31_STWX		  151
 #define	  PPC_31_WRTEEI		  163
+#define	  PPC_31_MTMSRD		  178
 #define	  PPC_31_STDUX		  181
 #define	  PPC_31_STWUX		  183
 #define	  PPC_31_SUBFZE		  200
@@ -129,11 +132,13 @@
 #define	  PPC_31_ADD		  266
 #define	  PPC_31_DCBT		  278
 #define	  PPC_31_LHZX		  279
+#define	  PPC_31_EQV		  284
 #define	  PPC_31_TLBIE		  306
 #define	  PPC_31_LHZUX		  311
 #define	  PPC_31_XOR		  316
 #define	  PPC_31_MFSPR		  339
 #define	  PPC_31_LHAX		  343
+#define	  PPC_31_LVXL		  359
 #define	  PPC_31_TLBIA		  370
 #define	  PPC_31_MFTB		  371
 #define	  PPC_31_LHAUX		  375
@@ -179,6 +184,7 @@
 #define	  PPC_31_ADDO		  778
 #define	  PPC_31_LHBRX		  790
 #define	  PPC_31_SRAW		  792
+#define	  PPC_31_DSSALL		  822
 #define	  PPC_31_SRAWI		  824
 #define	  PPC_31_EIEIO		  854
 #define	  PPC_31_TLBSX_DOT	  914
