@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: machine.c,v 1.679 2006/07/16 13:32:25 debug Exp $
+ *  $Id: machine.c,v 1.689 2006/10/04 11:56:40 debug Exp $
  */
 
 #include <stdio.h>
@@ -49,6 +49,7 @@
 #include "memory.h"
 #include "misc.h"
 #include "net.h"
+#include "settings.h"
 #include "symbol.h"
 
 
@@ -77,7 +78,7 @@ struct machine *machine_new(char *name, struct emul *emul)
 
 	memset(m, 0, sizeof(struct machine));
 
-	/*  Back pointer:  */
+	/*  Pointer back to the emul object that this machine belongs to:  */
 	m->emul = emul;
 
 	m->name = strdup(name);
@@ -93,7 +94,6 @@ struct machine *machine_new(char *name, struct emul *emul)
 	m->byte_order_override = NO_BYTE_ORDER_OVERRIDE;
 	m->boot_kernel_filename = "";
 	m->boot_string_argument = NULL;
-	m->automatic_clock_adjustment = 1;
 	m->x11_scaledown = 1;
 	m->x11_scaleup = 1;
 	m->n_gfx_cards = 1;
@@ -101,10 +101,60 @@ struct machine *machine_new(char *name, struct emul *emul)
 	m->show_symbolic_register_names = 1;
 	symbol_init(&m->symbol_context);
 
+	/*  Settings:  */
+	m->settings = settings_new();
+	settings_add(m->settings, "name", 0,
+	    SETTINGS_TYPE_STRING, SETTINGS_FORMAT_STRING,
+	    (void *) &m->name);
+	settings_add(m->settings, "serial_nr", 0,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_DECIMAL,
+	    (void *) &m->serial_nr);
+	settings_add(m->settings, "arch_pagesize", 0,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_DECIMAL,
+	    (void *) &m->arch_pagesize);
+	settings_add(m->settings, "prom_emulation", 0,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_YESNO,
+	    (void *) &m->prom_emulation);
+	settings_add(m->settings, "allow_instruction_combinations", 0,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_YESNO,
+	    (void *) &m->allow_instruction_combinations);
+	settings_add(m->settings, "n_gfx_cards", 0,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_DECIMAL,
+	    (void *) &m->n_gfx_cards);
+	settings_add(m->settings, "show_symbolic_register_names", 1,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_YESNO,
+	    (void *) &m->show_symbolic_register_names);
+	settings_add(m->settings, "statistics_enabled", 1,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_YESNO,
+	    (void *) &m->statistics_enabled);
+
 	return m;
 }
 
 
+/*
+ *  machine_destroy():
+ *
+ *  Destroys a machine object.
+ */
+void machine_destroy(struct machine *machine)
+{
+	int i;
+
+	for (i=0; i<machine->ncpus; i++)
+		cpu_destroy(machine->cpus[i]);
+
+	if (machine->name != NULL)
+		free(machine->name);
+
+	/*  Remove any remaining level-1 settings:  */
+	settings_remove_all(machine->settings);
+	settings_destroy(machine->settings);
+
+	free(machine);
+}
+
+
 /*
  *  machine_name_to_type():
  *
@@ -230,8 +280,8 @@ void machine_add_tickfunction(struct machine *machine, void (*func)
 	if (!machine->cycle_accurate) {
 		/*
 		 *  The dyntrans subsystem wants to run code in relatively
-		 *  large chunks without checking for external interrupts,
-		 *  so we cannot allow too low tickshifts:
+		 *  large chunks without checking for external interrupts;
+		 *  too low tickshifts are not allowed.
 		 */
 		if (tickshift < N_SAFE_DYNTRANS_LIMIT_SHIFT) {
 			fatal("ERROR! tickshift = %i, less than "
@@ -400,13 +450,6 @@ void machine_dumpinfo(struct machine *m)
 		debug(", dbe_on_nonexistant_memaccess");
 	debug("\n");
 
-	debug("clock: ");
-	if (m->automatic_clock_adjustment)
-		debug("adjusted automatically");
-	else
-		debug("fixed at %i Hz", m->emulated_hz);
-	debug("\n");
-
 	if (!m->prom_emulation)
 		debug("PROM emulation disabled\n");
 
@@ -583,8 +626,11 @@ int store_64bit_word(struct cpu *cpu, uint64_t addr, uint64_t data64)
 int store_32bit_word(struct cpu *cpu, uint64_t addr, uint64_t data32)
 {
 	unsigned char data[4];
+
+	/*  TODO: REMOVE THIS once everything is more stable!  */
 	if (cpu->machine->arch == ARCH_MIPS && (addr >> 32) == 0)
 		addr = (int64_t)(int32_t)addr;
+
 	data[0] = (data32 >> 24) & 255;
 	data[1] = (data32 >> 16) & 255;
 	data[2] = (data32 >> 8) & 255;
@@ -608,8 +654,11 @@ int store_32bit_word(struct cpu *cpu, uint64_t addr, uint64_t data32)
 int store_16bit_word(struct cpu *cpu, uint64_t addr, uint64_t data16)
 {
 	unsigned char data[2];
+
+	/*  TODO: REMOVE THIS once everything is more stable!  */
 	if (cpu->machine->arch == ARCH_MIPS && (addr >> 32) == 0)
 		addr = (int64_t)(int32_t)addr;
+
 	data[0] = (data16 >> 8) & 255;
 	data[1] = (data16) & 255;
 	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
@@ -629,6 +678,7 @@ void store_buf(struct cpu *cpu, uint64_t addr, char *s, size_t len)
 {
 	size_t psize = 1024;	/*  1024 256 64 16 4 1  */
 
+	/*  TODO: REMOVE THIS once everything is more stable!  */
 	if (cpu->machine->arch == ARCH_MIPS && (addr >> 32) == 0)
 		addr = (int64_t)(int32_t)addr;
 
@@ -672,18 +722,46 @@ void store_pointer_and_advance(struct cpu *cpu, uint64_t *addrp,
 }
 
 
+/*
+ *  load_64bit_word():
+ *
+ *  Helper function. Emulated byte order is taken into account.
+ */
+uint64_t load_64bit_word(struct cpu *cpu, uint64_t addr)
+{
+	unsigned char data[8];
+
+	cpu->memory_rw(cpu, cpu->mem,
+	    addr, data, sizeof(data), MEM_READ, CACHE_DATA);
+
+	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
+		int tmp = data[0]; data[0] = data[7]; data[7] = tmp;
+		tmp = data[1]; data[1] = data[6]; data[6] = tmp;
+		tmp = data[2]; data[2] = data[5]; data[5] = tmp;
+		tmp = data[3]; data[3] = data[4]; data[4] = tmp;
+	}
+
+	return
+	    ((uint64_t)data[0] << 56) + ((uint64_t)data[1] << 48) +
+	    ((uint64_t)data[2] << 40) + ((uint64_t)data[3] << 32) +
+	    ((uint64_t)data[4] << 24) + ((uint64_t)data[5] << 16) +
+	    ((uint64_t)data[6] << 8) + (uint64_t)data[7];
+}
+
+
 /*
  *  load_32bit_word():
  *
- *  Helper function.  Prints a warning and returns 0, if the read failed.
- *  Emulated byte order is taken into account.
+ *  Helper function. Emulated byte order is taken into account.
  */
 uint32_t load_32bit_word(struct cpu *cpu, uint64_t addr)
 {
 	unsigned char data[4];
 
+	/*  TODO: REMOVE THIS once everything is more stable!  */
 	if (cpu->machine->arch == ARCH_MIPS && (addr >> 32) == 0)
 		addr = (int64_t)(int32_t)addr;
+
 	cpu->memory_rw(cpu, cpu->mem,
 	    addr, data, sizeof(data), MEM_READ, CACHE_DATA);
 
@@ -699,15 +777,16 @@ uint32_t load_32bit_word(struct cpu *cpu, uint64_t addr)
 /*
  *  load_16bit_word():
  *
- *  Helper function.  Prints a warning and returns 0, if the read failed.
- *  Emulated byte order is taken into account.
+ *  Helper function. Emulated byte order is taken into account.
  */
 uint16_t load_16bit_word(struct cpu *cpu, uint64_t addr)
 {
 	unsigned char data[2];
 
+	/*  TODO: REMOVE THIS once everything is more stable!  */
 	if (cpu->machine->arch == ARCH_MIPS && (addr >> 32) == 0)
 		addr = (int64_t)(int32_t)addr;
+
 	cpu->memory_rw(cpu, cpu->mem,
 	    addr, data, sizeof(data), MEM_READ, CACHE_DATA);
 
@@ -851,10 +930,6 @@ void machine_setup(struct machine *machine)
 		debug(" (%.2f MHz)", (float)machine->emulated_hz / 1000000);
 	debug("\n");
 
-	/*  Default fake speed: 5 MHz  */
-	if (machine->emulated_hz < 1)
-		machine->emulated_hz = 5000000;
-
 	if (machine->bootstr != NULL) {
 		debug("bootstring%s: %s", (machine->bootarg!=NULL &&
 		    strlen(machine->bootarg) >= 1)? "(+bootarg)" : "",
