@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_algor.c,v 1.8 2006/08/14 18:03:12 debug Exp $
+ *  $Id: machine_algor.c,v 1.11 2006/09/26 08:49:18 debug Exp $
  */
 
 #include <stdio.h>
@@ -45,6 +45,8 @@
 
 MACHINE_SETUP(algor)
 {
+	machine->emulated_hz = 166560000;
+
 	switch (machine->machine_subtype) {
 	case MACHINE_ALGOR_P4032:
 		machine->machine_name = "Algor P4032";
@@ -81,9 +83,6 @@ MACHINE_SETUP(algor)
 	if (!machine->prom_emulation)
 		return;
 
-	/*  Magic "reboot" instruction at 0xbfc00000:  */
-	store_32bit_word(cpu, 0xffffffff9fc00000ULL, 0x00c0de0d);
-
 	/*  NetBSD/algor wants these:  */
 
 	/*  a0 = argc  */
@@ -100,7 +99,7 @@ MACHINE_SETUP(algor)
 	store_string(cpu, (int32_t)0x9fc01040, machine->bootstr);
 	store_string(cpu, (int32_t)0x9fc01200, machine->bootarg);
 
-	/*  a2 = (yamon_env_var *)envp  */
+	/*  a2 = pointer to environment strings  */
 	cpu->cd.mips.gpr[MIPS_GPR_A2] = (int32_t)0x9fc01800;
 	{
 		char tmps[50];
