@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_alpha.c,v 1.7 2006/06/24 10:19:19 debug Exp $
+ *  $Id: machine_alpha.c,v 1.11 2006/09/01 11:39:50 debug Exp $
  */
 
 #include <stdio.h>
@@ -49,11 +49,15 @@ MACHINE_SETUP(alpha)
 	struct crb crb;
 	struct ctb ctb;
 	struct mddt mddt;
+	struct pcs *pcs = malloc(sizeof(struct pcs) * machine->ncpus);
+	int i;
 
 	switch (machine->machine_subtype) {
 
 	case ST_ALPHABOOK1:
 		machine->machine_name = "AlphaBook 1";
+		if (machine->emulated_hz == 0)
+			machine->emulated_hz = 233000000;
 		device_add(machine, "lca");
 		break;
 
@@ -87,7 +91,7 @@ MACHINE_SETUP(alpha)
 	/*  a4 = Bootinfo version  */
 	cpu->cd.alpha.r[ALPHA_A0] = 16*1024*1024 / 8192;
 	cpu->cd.alpha.r[ALPHA_A1] = 0;	/*  TODO  */
-	cpu->cd.alpha.r[ALPHA_A2] = ALPHA_BOOTINFO_MAGIC;
+	cpu->cd.alpha.r[ALPHA_A2] = 0;	/*  Note: NOT ALPHA_BOOTINFO_MAGIC  */
 	cpu->cd.alpha.r[ALPHA_A3] = 0;	/*  TODO  */
 	cpu->cd.alpha.r[ALPHA_A4] = 1;
 
@@ -98,16 +102,25 @@ MACHINE_SETUP(alpha)
 	 */
 	memset(&rpb, 0, sizeof(struct rpb));
 	store_64bit_word_in_host(cpu, (unsigned char *)
-	    &(rpb.rpb_phys), HWRPB_ADDR);
+	    &(rpb.rpb_phys), 0x14000);
 	strlcpy((char *)&(rpb.rpb_magic), "HWRPB", 8);
 	store_64bit_word_in_host(cpu, (unsigned char *)
 	    &(rpb.rpb_size), sizeof(struct rpb));
 	store_64bit_word_in_host(cpu, (unsigned char *)
 	    &(rpb.rpb_page_size), 8192);
+	strlcpy((char *)&(rpb.rpb_ssn), "123456789", 10);
 	store_64bit_word_in_host(cpu, (unsigned char *)
 	    &(rpb.rpb_type), machine->machine_subtype);
 	store_64bit_word_in_host(cpu, (unsigned char *)
-	    &(rpb.rpb_cc_freq), 100000000);
+	    &(rpb.rpb_cc_freq), machine->emulated_hz);
+	store_64bit_word_in_host(cpu, (unsigned char *)
+	    &(rpb.rpb_intr_freq), 1024 << 12);
+	store_64bit_word_in_host(cpu, (unsigned char *)
+	    &(rpb.rpb_pcs_cnt), machine->ncpus);
+	store_64bit_word_in_host(cpu, (unsigned char *)
+	    &(rpb.rpb_pcs_size), sizeof(struct pcs));
+	store_64bit_word_in_host(cpu, (unsigned char *)
+	    &(rpb.rpb_pcs_off), PCS_ADDR - HWRPB_ADDR);
 	store_64bit_word_in_host(cpu, (unsigned char *)
 	    &(rpb.rpb_ctb_off), CTB_ADDR - HWRPB_ADDR);
 	store_64bit_word_in_host(cpu, (unsigned char *)
@@ -125,10 +138,21 @@ MACHINE_SETUP(alpha)
 	memset(&crb, 0, sizeof(struct crb));
 	store_64bit_word_in_host(cpu, (unsigned char *)
 	    &(crb.crb_v_dispatch), CRB_ADDR - 0x100);
-	store_64bit_word(cpu, CRB_ADDR - 0x100 + 8, 0x10000);
+	store_64bit_word(cpu, CRB_ADDR - 0x100 + 8, PROM_ENTRY_PADDR);
 	store_64bit_word_in_host(cpu, (unsigned char *)
 	    &(crb.crb_v_fixup), CRB_ADDR - 0x80);
-	store_64bit_word(cpu, CRB_ADDR - 0x80 + 8, 0x10800);
+	store_64bit_word(cpu, CRB_ADDR - 0x80 + 8, PROM_ENTRY_PADDR + 0x800);
+
+	/*  PCS: Processor ID etc.  */
+	for (i=0; i<machine->ncpus; i++) {
+		memset(&pcs[i], 0, sizeof(struct pcs));
+		store_64bit_word_in_host(cpu, (unsigned char *)
+		    &(pcs[i].pcs_flags), PCS_RC | PCS_PA | PCS_PP |
+		    PCS_CV | PCS_PV | PCS_PMV | PCS_PL);
+		store_64bit_word_in_host(cpu, (unsigned char *)
+		    &(pcs[i].pcs_proc_type),
+		    machine->cpus[i]->cd.alpha.cpu_type.pcs_type);
+	}
 
 	/*
 	 *  MDDT: Memory Data Descriptor Table. For now, it is a simple
@@ -139,30 +163,38 @@ MACHINE_SETUP(alpha)
 	 *  and similar.
 	 */
 	memset(&mddt, 0, sizeof(struct mddt));
+	memset(&mddt.mddt_clusters[0], 0, sizeof(struct mddt_cluster));
+	memset(&mddt.mddt_clusters[1], 0, sizeof(struct mddt_cluster));
 	store_64bit_word_in_host(cpu, (unsigned char *)
 	    &(mddt.mddt_cluster_cnt), 2);
 	store_64bit_word_in_host(cpu, (unsigned char *)
-	    &(mddt.mddt_clusters[0].mddt_pfn), 128 * 16);
+	    &(mddt.mddt_clusters[0].mddt_pfn), 16 * 128);
 	store_64bit_word_in_host(cpu, (unsigned char *)
 	    &(mddt.mddt_clusters[0].mddt_pg_cnt),
-	    (machine->physical_ram_in_mb - 16) * 128);
+	    (machine->physical_ram_in_mb/2 - 16) * 128);
 	store_64bit_word_in_host(cpu, (unsigned char *)
-	    &(mddt.mddt_clusters[1].mddt_pfn), machine->physical_ram_in_mb*128);
+	    &(mddt.mddt_clusters[1].mddt_pfn),
+	    machine->physical_ram_in_mb/2 * 128);
 	store_64bit_word_in_host(cpu, (unsigned char *)
-	    &(mddt.mddt_clusters[1].mddt_pg_cnt), (machine->physical_ram_in_mb
-	    - 1) * 128);
+	    &(mddt.mddt_clusters[1].mddt_pg_cnt),
+	    (machine->physical_ram_in_mb/2) * 128);
 
 	/*
-	 *  Place a special "hack" palcode call at 0x10000 and 0x10800:
+	 *  Place a special "hack" palcode call at PROM_ENTRY_PADDR and
+	 *  PROM_ENTRY_PADDR + 0x800:
 	 *  (Hopefully nothing else will be there.)
 	 */
-	store_32bit_word(cpu, 0x10000, 0x3fffffe);
-	store_32bit_word(cpu, 0x10800, 0x3fffffd);
+	store_32bit_word(cpu, PROM_ENTRY_PADDR, 0x3fffffe);
+	store_32bit_word(cpu, PROM_ENTRY_PADDR + 0x800, 0x3fffffd);
 
 	store_buf(cpu, HWRPB_ADDR, (char *)&rpb, sizeof(struct rpb));
 	store_buf(cpu, CTB_ADDR, (char *)&ctb, sizeof(struct ctb));
 	store_buf(cpu, CRB_ADDR, (char *)&crb, sizeof(struct crb));
 	store_buf(cpu, MEMDAT_ADDR, (char *)&mddt, sizeof(struct mddt));
+	store_buf(cpu, PCS_ADDR, (char *)pcs, sizeof(struct pcs) *
+	    machine->ncpus);
+
+	free(pcs);
 }
 
 
@@ -195,7 +227,7 @@ MACHINE_DEFAULT_CPU(alpha)
 
 MACHINE_DEFAULT_RAM(alpha)
 {
-	machine->physical_ram_in_mb = 64;
+	machine->physical_ram_in_mb = 128;
 }
 
 
