@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_hpcsh.c,v 1.2 2006/06/24 10:19:19 debug Exp $
+ *  $Id: machine_hpcsh.c,v 1.6 2006/10/19 10:15:57 debug Exp $
  */
 
 #include <stdio.h>
@@ -64,12 +64,38 @@ MACHINE_SETUP(hpcsh)
 */
 	machine->machine_name = "HPCsh";
 	cpu->byte_order = EMUL_LITTLE_ENDIAN;
+
+	if (!machine->use_x11)
+		fprintf(stderr, "-------------------------------------"
+		    "------------------------------------------\n"
+		    "\n  WARNING!  You are emulating a Dreamcast without -X."
+		    "\n            You will miss graphical output!\n\n"
+		    "-------------------------------------"
+		    "------------------------------------------\n");
+
+	/*  32 MB in two parts, each included twice (shadowed):  */
+	dev_ram_init(machine, 0x0c000000, 0x01000000, DEV_RAM_MIRROR, 0x0);
+	dev_ram_init(machine, 0x0d000000, 0x01000000, DEV_RAM_MIRROR, 0x0);
+	dev_ram_init(machine, 0x0e000000, 0x01000000, DEV_RAM_MIRROR,
+	    0x01000000);
+	dev_ram_init(machine, 0x0f000000, 0x01000000, DEV_RAM_MIRROR,
+	    0x01000000);
+
+	dev_fb_init(machine, machine->memory, 0x10000000,
+	    VFB_HPC, 640,240, 640,240, 16, machine->machine_name);
 }
 
 
 MACHINE_DEFAULT_CPU(hpcsh)
 {
-	machine->cpu_name = strdup("SH");
+	machine->cpu_name = strdup("SH7750");
+}
+
+
+MACHINE_DEFAULT_RAM(hpcsh)
+{
+	/*  TODO: Model dependent. */
+	machine->physical_ram_in_mb = 32;
 }
 
 
@@ -84,5 +110,7 @@ MACHINE_REGISTER(hpcsh)
 
 	machine_entry_add_subtype(me, "Jornada 690",
 	    MACHINE_HPCSH_JORNADA690, "jornada690", NULL);
+
+	me->set_default_ram = machine_default_ram_hpcsh;
 }
 
